#include <sys/types.h>
#include <sys/stat.h>
 
#include <glib-object.h>
#include <glib.h>

#include <gtk/gtk.h>
#include "interface.h"
#include "remote-udp.h"
#include "keyvalues.h"

#ifdef HAVE_NTABLET
 #include <libosso.h>
 #define DEFAULT_GTKRC "gtkrc.tablet"
#else
 #define DEFAULT_GTKRC "gtkrc"
#endif

#ifdef HAVE_CONFIG_H
 #include <config.h>
#endif

#define HOME_CONFIG ".lcontrol"

int main (int argc, char *argv[])
{
  GtkWidget *window = NULL;
  struct stat sb;

  RemoteUdp *udp = NULL;
  gint retval = 0;
  guint port = 0;
  gchar *host = NULL;
  const gchar *homedir = g_get_home_dir ();
  gchar *configdir = NULL;
  gchar *gtkrcfile = NULL;
#ifdef HAVE_NTABLET
  osso_context_t *osso;

  g_print("package: %s v:%s", PACKAGE, VERSION);
  osso = osso_initialize(PACKAGE, VERSION, FALSE, NULL);
  if (osso == NULL) {
    g_error("Failed to initialize libosso");
    return FALSE;
  }
#endif

  gtk_init (&argc, &argv);

  if (homedir == NULL) {
    g_critical ("Could not get home directory");
    return 1;
  }
  
  gtkrcfile = g_build_filename (homedir, HOME_CONFIG, DEFAULT_GTKRC, NULL);
  if (gtkrcfile == NULL) {
    g_critical ("Could not build gtkrcfile name");
    return 1;
  }
  g_print ("gtkrc (home): %s\n", gtkrcfile);

  retval = stat(gtkrcfile, &sb);
  if(retval != 0) {
    g_free (gtkrcfile);
    gtkrcfile = g_build_filename (DATA_DIR, PACKAGE, DEFAULT_GTKRC, NULL);
    if (gtkrcfile == NULL) {
      g_critical ("Could not build gtkrcfile name from datadir");
      return 1;
    }
    g_print ("gtkrc (system): %s\n", gtkrcfile);
    retval = stat(gtkrcfile, &sb);
    if(retval != 0) {
      g_print ("no gtkrcfile found\n");
      return 2;
    }
  }

  g_print ("gtkrc: %s\n", gtkrcfile);
  gtk_rc_parse(gtkrcfile);


  configdir = g_build_filename (homedir, HOME_CONFIG, "config", NULL);
  if (configdir == NULL) {
    g_critical ("Could not build configfile name");
    return 1;
  }
  g_print ("config (home): %s\n", configdir);
  retval = stat(configdir, &sb);
  if(retval != 0) {
    g_free (configdir);
    configdir = g_build_filename (DATA_DIR, PACKAGE, "config", NULL);
    if (configdir == NULL) {
      g_critical ("Could not build config filename from datadir");
      return 1;
    }
    g_print ("config (system): %s\n", configdir);
    retval = stat(configdir, &sb);
    if(retval != 0) {
      g_print ("no configfile found\n");
      return 2;
    }
  }

  g_print ("config directory: %s\n", configdir);

  retval = read_keyfile (configdir);
  g_free (configdir);

  if (retval != 0) {
    g_critical ("Could not init configure\n");
    free_keyfile ();
    return 1;
  }

  port = get_port ();
  host = g_strdup (get_host ());

  g_print ("host='%s' port='%d'\n", host, port);

  udp = remote_udp_new ();
  if (udp == NULL) {
    g_critical ("Could not create remote udp object\n");
    g_free (host);
    free_keyfile ();
    return -1;
  }
  g_object_set (G_OBJECT(udp), 
		"host", host,
		"port", port,
		NULL);

  set_remote_udp (udp);

  window = create_main_ui ();
  if (window == NULL) {
    g_object_unref (udp);
    g_free (host);
    free_keyfile ();
    g_critical ("Could not create window\n");
    return 1;
  }

  gtk_widget_show (window);
  gtk_main ();

  free_keyfile ();

  g_object_unref (udp);
  g_free (host);


#ifdef HAVE_NTABLET
  osso_deinitialize(osso);
#endif

  return 0;
}
