#include "rcbutton.h"

static void rc_button_class_init (RcButtonClass *klass);
static void rc_button_init (RcButton *button);
static void rc_button_destroy (GtkObject *object);

static GObject *rc_button_constructor (GType type,
				       guint n_construct_properties,
				       GObjectConstructParam *construct_params);
static void rc_button_set_property (GObject *object, guint prop_id, 
				    const GValue *value, GParamSpec *pspec);
static void rc_button_get_property (GObject *object, guint prop_id, 
				    GValue *value, GParamSpec *pspec);

enum {
  PROP_0,
  PROP_DATA
};

G_DEFINE_TYPE (RcButton, rc_button, GTK_TYPE_BUTTON)

static void
rc_button_class_init (RcButtonClass *klass)
{
  GObjectClass *gobject_class;
  GtkObjectClass *object_class;
  GtkWidgetClass *widget_class;
  GtkContainerClass *container_class;

  gobject_class = G_OBJECT_CLASS (klass);
  object_class = (GtkObjectClass*) klass;
  widget_class = (GtkWidgetClass*) klass;
  container_class = (GtkContainerClass*) klass;
  
  gobject_class->constructor = rc_button_constructor;
  gobject_class->set_property = rc_button_set_property;
  gobject_class->get_property = rc_button_get_property;

  object_class->destroy = rc_button_destroy;
  g_object_class_install_property (gobject_class, PROP_DATA,
                                   g_param_spec_pointer ("data",
                                                        "Data",
                                                        "Data of the button",
                                                        G_PARAM_READWRITE | 
                                                         G_PARAM_CONSTRUCT));

}

static void
rc_button_init (RcButton *button)
{
  button->data = NULL;
}

static void
rc_button_destroy (GtkObject *object)
{
  (* GTK_OBJECT_CLASS (rc_button_parent_class)->destroy) (object);
}

static GObject *
rc_button_constructor (GType type,
		       guint n_construct_properties,
		       GObjectConstructParam *construct_params)
{
  GObject *object;

  object = (* G_OBJECT_CLASS (rc_button_parent_class)->constructor) (type,
								      n_construct_properties,
								      construct_params);
  return object;
}

static void
rc_button_set_property (GObject *object, guint prop_id, const GValue *value,
                         GParamSpec *pspec)
{
  RcButton *button = RC_BUTTON (object);

  switch (prop_id)
    {
    case PROP_DATA:
      button->data = (gpointer)g_value_get_pointer (value);
      break;
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

static void
rc_button_get_property (GObject *object, guint prop_id, GValue *value, 
			 GParamSpec *pspec)
{
  RcButton *button = RC_BUTTON (object);

  switch (prop_id)
    {
    case PROP_DATA:
      g_value_set_pointer (value, (gpointer)button->data);
      break;
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}



GtkWidget *
rc_button_new (void)
{
  return g_object_new (RC_TYPE_BUTTON, NULL);
}

GtkWidget *
rc_button_new_with_data (gpointer data)
{
  return g_object_new (RC_TYPE_BUTTON, "data", data, NULL);
}

